-- debug helpers

function Bikinitools:getDiagString()
	local str = "";

	if isAdmin()
	then
		str = str .. "a";
	end

	if getCore():getDebug()
	then
		str = str .. "d";
	end

	if isCoopHost()
	then
		str = str .. "ch";
	end

	if isClient()
	then
		str = str .. "c";
	end

	--if getAccessLevel() ~= ""
	--then
		--str = str .. "" .. getAccessLevel() .. "";
	--end

	--str = str .. " (" .. Bikinitools.Options:getSettingsFile() .. ")";

	if str ~= ""
	then
		str = "|" .. str;
	end

	return Bikinitools:ternary(
		Bikinitools:playerIsAdmin(),
		"A",
		"C"
	) .. str;
end


function Bikinitools:message(message, silent)
    message = Bikinitools:printValue(message);

	if not (Bikinitools.Options.silentMode or silent == true)
	then
		local player = Bikinitools:getPlayer();

        if player
        then
            local r = 0.25;
            local g = 0.25;
            local b = 0.25;

            if Bikinitools.Config
            then
                r = Bikinitools.Config.colors.highlightR;
                g = Bikinitools.Config.colors.highlightG;
                b = Bikinitools.Config.colors.highlightB;
            end

            --player:Say(message, r, g, b, UIFont.Small, 30.0, "radio");
            --player:setHaloNote(string.format("*%s,%s,%s*%s", r, g, b, message));

            table.insert(Bikinitools.messageQueue, function()
                player:setHaloNote(message, r * 255, g * 255, b * 255, 300);
            end);
        end
	end

    print(message);
end


function Bikinitools:printValue(value)
    local vType = type(value)

    if vType == "boolean"
    then
        if value == true
        then
            return "true";
        else
            return "false";
        end
    elseif vType == "function"
    then
        return "<function>";
    elseif not value
    then
        return "";
    end

    return tostring(value) or "[?]";
end


function Bikinitools:printKeyValuePairs(array, title)
    Bikinitools:message(title or "Bikinitools:printKeyValuePairs()", true);

    for varName, value in pairs(array)
    do
        Bikinitools:message(" - " .. (varName or "") .. " = " .. Bikinitools:printValue(value), true);
    end
end


function Bikinitools:printArray(array, title)
    Bikinitools:message(title or "Bikinitools:printArray()", true);

    for i, value in ipairs(array)
    do
        Bikinitools:message(" - " .. i .. " = " .. Bikinitools:printValue(value), true);
    end
end


function Bikinitools:printArrayWithSimpleTableItems(array, title)
    Bikinitools:message(title or "Bikinitools:printArrayWithSimpleTableItems()", true);

    for i, values in ipairs(array)
    do
        local outStr = "";

        for vI, value in ipairs(values)
        do
            outStr = outStr .. vI .."={" .. value .. "} ";
        end

        Bikinitools:message(" - " .. outStr, true);
    end
end


function Bikinitools:printSquareCoords(square, returnString)
	local str = square:getX() .. "|" .. square:getY();

	if not returnString
	then
		Bikinitools:message(str);
	else
		return str;
	end
end


function Bikinitools:printModData(object)
	if object
	then
		local data = object:getModData();

		if data
		then
			Bikinitools:printKeyValuePairs(data, "printing modData");
		end
	end
end