-- crutch for moddata


function Bikinitools.Vehicle:getModData(vehicle)
	local part = Bikinitools.Vehicle:getMulePart(vehicle);

	if part
	then
		return part:getModData();
	else
		--Bikinitools:message("data mule part NOT found");

		return {};
	end
end


function Bikinitools.Vehicle:saveModData(vehicle, data)
	data["_vehicleId"] = vehicle:getId();

	Bikinitools:sendClientCommand("bikinitools", "setVehicleData", data);
end


-- vehicle specific helpers


function Bikinitools.Vehicle:refuel(vehicle)
    local tank = Bikinitools.Vehicle:getGasTank(vehicle);

    if tank
    then
		Bikinitools.VehiclePart:setContent(tank, tank:getContainerCapacity() * 0.5);
    end
end


function Bikinitools.Vehicle:get(player, silent)
    if not player
    then
        player = Bikinitools:getPlayer();
    end

    local vehicle = player:getVehicle();

    if vehicle == nil
    then
        vehicle = ISVehicleMenu.getVehicleToInteractWith(player);
    end

    if vehicle == nil
    then
        vehicle = Bikinitools.Vehicle.playerVehicle;
    end

    --if vehicle
    --then
        --Bikinitools:message("Vehicle found: " .. Bikinitools.Vehicle:getName(vehicle), silent);

		--Bikinitools:printModData(vehicle);
    --else
        --Bikinitools:message("No vehicle found", silent);
    --end

    return vehicle;
end


function Bikinitools.Vehicle:getName(vehicle)
    if type(vehicle) ~= "string"
    then
        return ISVehicleMenu.getVehicleDisplayName(vehicle);
    else
        local name = getText("IGUI_VehicleName" .. vehicle);

        if string.match(vehicle, "Burnt")
        then
            local unburnt = string.gsub(vehicle, "Burnt", "");

            if getTextOrNull("IGUI_VehicleName" .. unburnt)
            then
                name = getText("IGUI_VehicleName" .. unburnt);
            end

            name = getText("IGUI_VehicleNameBurntCar", name);
        end

        return name;
    end
end


function Bikinitools.Vehicle:repair(vehicle)
    if vehicle
    then
        for i = 0, vehicle:getPartCount() -1
        do
            local part = vehicle:getPartByIndex(i);

            if not Bikinitools.VehiclePart:isMissing(part)
            then
				Bikinitools.VehiclePart:repair(part);
            end
        end

        Bikinitools.Vehicle:refuel(vehicle);
    end
end


function Bikinitools.Vehicle:remove(vehicle)
    if vehicle
    then
		--if Bikinitools:playerIsAdmin()
		--then
			--vehicle:permanentlyRemove();
		--else
			Bikinitools:sendClientCommand("vehicle", "remove", {
				vehicle = vehicle:getId()
			});

			--Commands.remove(Bikinitools:getPlayer(), {
				--vehicle = vehicle:getId()
			--});
		--end
    end
end


-- search functions and checks


function Bikinitools.Vehicle:getFirstDoor(vehicle)
    return Bikinitools.VehiclePart:findBy(vehicle, function(part)
        if string.sub(part:getId(), 0, 4) == "Door" and not string.match(part:getId(), "Rear") and part:getDoor() and not part:getDoor():isLockBroken()
        then
            return part;
        end
    end) or false;
end


function Bikinitools.Vehicle:getGasTank(vehicle)
    return Bikinitools.VehiclePart:findBy(vehicle, function(part)
        if string.sub(part:getId(), 0, 3) == "Gas"
        then
            return part;
        end
    end) or false;
end


function Bikinitools.Vehicle:getTrunkDoor(vehicle)
	return Bikinitools.VehiclePart:findBy(vehicle, function(part)
        if (string.sub(part:getId(), 0, 5) == "Trunk" or string.match(part:getId(), "DoorRear")) and part:getDoor() and not part:getDoor():isLockBroken()
        then
            return part;
        end
    end)
end


-- context menu helpers


function Bikinitools.Vehicle:createSpecificVehicleContextMenu(context, player, vehicle)
    if vehicle and (
        Bikinitools.Options.enableSkinSwitcher
        or Bikinitools.Options.enableOpenSesame
        or Bikinitools.Options.enableGetKeyContext
        or Bikinitools.Options.enableRepairContext
        or Bikinitools.Options.enableVehicleRemover
        or Bikinitools.Options.enableTrunkUnlocker
    ) then
        Bikinitools:addContextMenuHeader(context, Bikinitools.Vehicle:getName(vehicle));

        if Bikinitools.Options.enableOpenSesame
        then
            Bikinitools.Vehicle:addOpenSesameContextMenuEntry(context, player, vehicle);
        end

        if Bikinitools.Options.enableGetKeyContext
        then
            Bikinitools.Vehicle:addKeyCreatorContextMenuEntry(context, player, vehicle);
        end

		if Bikinitools.Options.enableTrunkUnlocker
		then
			Bikinitools.Vehicle:addTrunkUnlockerContextMenuEntry(context, player, vehicle)
		end

        if Bikinitools.Options.enableRepairContext
        then
            Bikinitools.Vehicle:addRepairContextMenuEntry(context, player, vehicle);
        end

        if Bikinitools.Options.enableSkinSwitcher and not player:getVehicle()
        then
            Bikinitools.Vehicle:addSkinContextMenuEntry(context, player, vehicle);
        end

        if Bikinitools.Options.enableVehicleRemover and not player:getVehicle()
        then
            Bikinitools.Vehicle:addRemoveContextMenuEntry(context, player, vehicle);
        end
    end
end


function Bikinitools.Vehicle:createVehicleContextMenu(context, player)
    if Bikinitools.Options.enableVehicleSpawner
        or Bikinitools.Options.enableCellVehicleRemover
		or Bikinitools.Options.enableVehiclePositionSaver
    then
        Bikinitools:addContextMenuHeader(context, getText("IGUI_bikinitools_O_Context_Vehicle"));

        if Bikinitools.Options.enableVehicleSpawner and not player:getVehicle()
        then
            Bikinitools.Vehicle:addVehicleSpawnerContextMenuEntry(context, player);
        end

        if Bikinitools.Options.enableCellVehicleRemover
        then
            Bikinitools.Vehicle:addRemoveInCellContextMenuEntry(context, player, vehicle);
        end

		if Bikinitools.Options.enableVehiclePositionSaver
		then
			Bikinitools.Vehicle:addPositionSaverContextMenuEntry(context, player, vehicle);
		end
    end
end