-- sorter functions


function Bikinitools.Item.sortFnMoreFull(item1, items)
    local delta1 = 0;
    local delta2 = 0;

    if item1
    then
        delta1 = item1:getUsedDelta();
    end

    if item2
    then
        delta2 = item2:getUsedDelta();
    end

    return delta1 - delta2;
end


function Bikinitools.Item.sortFnLessFull(item1, items)
    local delta1 = 0;
    local delta2 = 0;

    if item1
    then
        delta1 = item1:getUsedDelta();
    end

    if item2
    then
        delta2 = item2:getUsedDelta();
    end

    return delta2 - delta1;
end


-- filter functions


function Bikinitools.Item.filterFnNotFull(item, fType)
    return (item:getFullType() == fType) and (item:getUsedDelta() + item:getUseDelta() <= 1);
end


function Bikinitools.Item.filterFnFull(item, fType)
    return (item:getFullType() == fType) and (item:getUsedDelta() + item:getUseDelta() > 1);
end


function Bikinitools.Item.filterFnAxe(item)
    return not item:isBroken() and item:hasTag("ChopTree");
end


function Bikinitools.Item.filterFnCutPlant(item)
    return not item:isBroken() and item:hasTag("CutPlant");
end


function Bikinitools.Item.filterFnCorpse(item)
    return item:getType() == "CorpseMale" or item:getType() == "CorpseFemale";
end


-- search functions


function Bikinitools.Item:findNonBrokenInInventoryByTag(player, tag)
    function searchFn(item)
        return not item:isBroken() and item:hasTag(tag);
    end

    return Bikinitools.Item:findInInventoryBy(player, searchFn);
end


function Bikinitools.Item:findEmptyContainerInInventoryByName(player, name)
    function searchFn(item)
        return item:getInventory():isEmpty();
    end

    return player:getInventory():getFirstTypeEvalRecurse(name, searchFn);
end


function Bikinitools.Item:findFillableInInventoryByName(player, name)
    return player:getInventory():getBestEvalArgRecurse(Bikinitools.Item.filterFnNotFull, Bikinitools.Item.sortFnMoreFull, name);
end


function Bikinitools.Item:findInInventoryBy(player, searchFn)
    return player:getInventory():getFirstEvalRecurse(searchFn);
end


function Bikinitools.Item:getPlantCuttingTool(player)
    return Bikinitools.Item:findNonBrokenInInventoryByTag(player, "CutPlant");
end


function Bikinitools.Item:getBestWoodChoppingAxe(player)
    local axes = player:getInventory():getAllEvalRecurse(Bikinitools.Item.filterFnAxe);
    local axe = nil;

    for i = 0, axes:size() - 1
    do
        if not axe or axe:getTreeDamage() < axes:get(i):getTreeDamage()
        then
            axe = axes:get(i);
        end
    end

    return axe;
end


function Bikinitools.Item:getFillableBagForSoil(player, soilType, preferEmpty)
    local fullType = Bikinitools.Item:getBagTypeToSoilType(soilType);
    local item = nil;

    if not preferEmpty
    then
        item = Bikinitools.Item:findFillableInInventoryByName(player, fullType);
    end

    if not item
    then
        item = Bikinitools.Item:findEmptyContainerInInventoryByName(player, "EmptySandbag");
    end

    return fullType,item;
end


function Bikinitools.Item:getFullSoilBag(player)
    local inventory = player:getInventory();

    for i, soilType in ipairs({"dirt", "sand", "gravel"})
    do
        local bags = inventory:getItemsFromFullType(Bikinitools.Item:getBagTypeToSoilType(soilType));

        for j = 0, bags:size() - 1
        do
            local bag = bags:get(j);

            if bag:getUsedDelta() > 0.75 and bag:getContainer() == inventory
            then
                return bag;
            end
        end
    end
end


function Bikinitools.Item:getBagTypeToSoilType(soilType)
    local fullType = nil;

    if soilType == "dirt" then
        fullType = "Base.Dirtbag"
    elseif soilType == "gravel" then
        fullType = "Base.Gravelbag"
    elseif soilType == "sand" then
        fullType = "Base.Sandbag"
    end

    return fullType;
end


-- container and inventory


function Bikinitools.Item:getPlayerContainers(player)
    return player:getInventory():getItemsFromCategory("Container");
end


function Bikinitools.Item:tryToPutItemAway(player, item, useQueue)
    local containers = Bikinitools.Item:getPlayerContainers(player);

    for i = 0, containers:size() - 1
    do
        local container = containers:get(i);

        --for i, item in ipairs(items)
        --do
            if container:isEquipped() and Bikinitools.Item:tryToPutItemInContainer(player, container, item, useQueue)
            then
                return true;
            end
        --end
    end
end


function Bikinitools.Item:tryToPutItemInContainer(player, container, item, useQueue)
    --Bikinitools:message("trying to put " .. item:getDisplayName() .. " into " .. container:getDisplayName());

    local container = container:getItemContainer();

    if container ~= nil and (container:getCapacityWeight() + item:getWeight()) <= container:getCapacity()
    then
        --Bikinitools:message(" - enough space in container");

        if not useQueue
        then
            container:addItem(item);
        else
            --javaTransferItems(player, item, item:getContainer(), container);
        end

        return true;
    else
        --Bikinitools:message(" - not enough space in container");

        container = player:getInventory();
    end

    return false;
end


function Bikinitools.Item:getFloorContainer(player)
	return ISInventoryPage.GetFloorContainer(player:getPlayerNum());
end


function Bikinitools.Item:pickUpFromFloor(player, itemType)
	local floorContainer = Bikinitools.Item:getFloorContainer(player);

	if floorContainer
	then
		local items = floorContainer:FindAll(itemType);

		if items
		then
			for i = 0, items:size() - 1
			do
				local item = items:get(i);

				ISTimedActionQueue.add(ISInventoryTransferAction:new(player, item, item:getContainer(), player:getInventory(), nil));
			end

			return items:size();
		end
	end
end


function Bikinitools.Item:getAllObjectContainers(object)
	local containers = {};

	for i = 0, object:getContainerCount() - 1
	do
		local container = object:getContainerByIndex(i);

		if container
		then
			table.insert(containers, container);
		end
	end

	return containers;
end


function Bikinitools.Item:emptyContainerToFloor(object, player)
	if object
	then
		local containers = Bikinitools.Item:getAllObjectContainers(object);

        for i = 1, #containers
        do
            local container = containers[i];

			if container and --luautils.walkToContainer(container, player:getPlayerNum())
				Bikinitools.Square:walkTo(player, object:getSquare(), true)
			then
				local items = container:getItems();

				for j = 0, items:size() - 1
				do
					local item = items:get(j);

					if not instanceof(item, "Moveable")
					then
						Bikinitools:message(getText("IGUI_bikinitools_I_DroppingX", item:getDisplayName()));

						ISInventoryPaneContextMenu.dropItem(item, player:getPlayerNum());
					end
				end
			end
        end
	end
end


-- specific helpers


function Bikinitools.Item:dropElectronicsScrapResults(player)
	local inventory = player:getInventory();
	local droppedCount = 0;

	for i, itemType in ipairs({
		"ElectronicsScrap", "Amplifier", "Battery", "ElectricWire", "MotionSensor", "RadioReceiver", "RadioTransmitter", "LightBulb", "LightBulbRed", "LightBulbGreen", "LightBulbBlue", "LightBulbYellow", "LightBulbCyan", "LightBulbMagenta", "LightBulbOrange", "LightBulbPurple", "LightBulbPink", "Aluminum"
	})
	do
		local items = inventory:FindAll(itemType);

		for i = 0, items:size() - 1
		do
			local item = items:get(i);

			ISInventoryPaneContextMenu.dropItem(item, player:getPlayerNum());
		end

		droppedCount = droppedCount + items:size();
	end

	return droppedCount;
end


function Bikinitools.Item:pickUpNailsFromFloor(player)
	return Bikinitools.Item:pickUpFromFloor(player, "Nails");
end


-- ideas

-- player.isSeeEveryone
-- getPlayer():setCanSeeAll(true)