-- helpers


function Bikinitools.Square:getCurrent(player)
    return getWorld():getCell():getGridSquare(player:getX(), player:getY(), player:getZ());
end


function Bikinitools.Square:highlight(square, state, r, g, b)
    local objects = square:getObjects();

    for i = 0, objects:size() - 1
    do
        local object = objects:get(i);
        local sprite = object:getSprite();

        if sprite and sprite:getName()
        then
			local spriteName = string.sub(sprite:getName(), 0, 7);

			if not object:isHighlighted() and (spriteName == "floors_" or spriteName == "blends_")
			then
				object:setHighlighted(state);
				object:setHighlightColor(
					r or Bikinitools.Config.colors.highlightR,
					g or Bikinitools.Config.colors.highlightG,
					b or Bikinitools.Config.colors.highlightB,
				1);
				--object:setOutlineHighlight(true);
				--object:setOutlineHighlightCol(
					--r or Bikinitools.Config.colors.highlightR,
					--g or Bikinitools.Config.colors.highlightG,
					--b or Bikinitools.Config.colors.highlightB,
				--1);
				--object:setOutlineThickness(0.5);
			end
        end
    end
end


function Bikinitools.Square:walkTo(player, square, synchronous)
	if square:getZ() < Bikinitools.Square:getCurrent(player):getZ()
	then
		setGameSpeed(1);
	end

	if synchronous
	then
		return luautils.walkAdj(player, square, true);
	else
		return ISTimedActionQueue.add(ISWalkToTimedAction:new(player, square));
	end
end


-- search functions


function Bikinitools.Square:getSoilType(square)
    return ISShovelGroundCursor.GetDirtGravelSand(square);
end


function Bikinitools.Square:getGrass(square)
    return Bikinitools.Square:checkObjectsForIsoFlag(square, "canBeRemoved");
end


function Bikinitools.Square:getPlant(square)
    return Bikinitools.Square:checkObjectsForIsoFlag(square, "canBeCut");
end


function Bikinitools.Square:getPotentiallyScrappableObjects(square, player)
	return ISMoveableTools.getScrapableObjects(player, square);
end


function Bikinitools.Square:getScrappableObjects(square, player, disableChecks)
	local scrappable = {};

	for i, scrapObject in ipairs(Bikinitools.Square:getPotentiallyScrappableObjects(square, player))
	do
		local resultScrap, chance, perkname = scrapObject.moveProps:canScrapObject(player);

		if disableChecks or (resultScrap.craftValid and resultScrap.canScrap)
		then
			table.insert(scrappable, {
				object = scrapObject.object,
				moveProps = scrapObject.moveProps
			});
		end
	end

	return scrappable;
end


function Bikinitools.Square:getFirstScrappableObject(square, player, allScrappables, returnObject)
	local scrapList = Bikinitools.Square:getScrappableObjects(square, player, allScrappables);

	if #scrapList > 0
	then
		if not returnObject
		then
			return scrapList[1];
		else
			return scrapList[1].object;
		end
	end

	return nil;
end


function Bikinitools.Square:getFilledContainerObjects(square)
	local containers = {};
	local objects = ISMoveableTools.getObjectList(square);

	for i = 1, #objects
	do
		local object = objects[i].object;
		local objContainers = Bikinitools.Item:getAllObjectContainers(object);

		for i = 1, #objContainers
		do
			if objContainers[i]:getContentsWeight() > 0
			then
				table.insert(containers, object);

				break;
			end
		end
	end

	return containers;
end


function Bikinitools.Square:getFirstFilledContainer(square)
	local containers = Bikinitools.Square:getFilledContainerObjects(square);

	if #containers > 0
	then
		return containers[1];
	end

	return nil;
end


function Bikinitools.Square:checkObjectsForIsoFlag(square, isoFlag)
    local objects = square:getObjects();

    for i = 0, objects:size() - 1
    do
        local object = objects:get(i);

        if object:getSprite() and object:getSprite():getProperties() and object:getSprite():getProperties():Is(IsoFlagType[isoFlag])
        then
            return square;
        end
    end

    return nil;
end


function Bikinitools.Square:getValidCorpses(square)
	local corpses = square:getDeadBodys();
	local result = {};

	for i = 0, corpses:size() - 1
	do
		local corpse = corpses:get(i);

		if corpse and corpse:getContainer():getContentsWeight() <= Bikinitools.Config.maxCorpseWeight
		then
			table.insert(result, corpse);
		end
	end

	return result;
end


-- specific tasks


function Bikinitools.Square:extractSoil(player, square)
    local shovel = Bikinitools.Item:findNonBrokenInInventoryByTag(player, "TakeDirt");

    if shovel
    then
        local soilType, soilTile = Bikinitools.Square:getSoilType(square);

        if soilType
        then
            local bagType, bag = Bikinitools.Item:getFillableBagForSoil(player, soilType);

            if bag
            then
                ISWorldObjectContextMenu.transferIfNeeded(player, bag);
                ISWorldObjectContextMenu.equip(player, player:getPrimaryHandItem(), shovel, true, true);

                if Bikinitools.Square:walkTo(player, square, true)
                then
                    ISTimedActionQueue.add(ISShovelGround:new(player, bag, soilTile, "blends_natural_01_64", bagType));

                    local fullBag = Bikinitools.Item:getFullSoilBag(player);

                    if fullBag -- and fullBag:getWeight() > 1.5
                    then
                        Bikinitools.Item:tryToPutItemAway(player, fullBag, false);
                    end
                end
            else
                Bikinitools:message(getText("IGUI_bikinitools_A_NoEmptyBag"));

                Bikinitools.Automation:stopGravelBuddy();
            end
        end
    end
end