-- config


Bikinitools.Options:register("lowPerformanceQueueMode", false, true);
Bikinitools.Options:register("predetermineAreaViability", false);
Bikinitools.Options:register("enableFunSizeRadii", false);


-- helpers


function Bikinitools.Automation:register(varName)
	Bikinitools.Automation[varName .. "Working"] = false;

	local needRestartVarName = "enable" .. Bikinitools:ucFirst(varName);

	if not Bikinitools:valueIsInArray(Bikinitools.Options.needRestart, needRestartVarName)
	then
		table.insert(Bikinitools.Options.needRestart, needRestartVarName);
	end

	if not Bikinitools:valueIsInArray(Bikinitools.Automation.queueLockVars, varName)
	then
		table.insert(Bikinitools.Automation.queueLockVars, varName);
	end
end


function Bikinitools.Automation:isAnyAutomationEnabled()
    for i, lockVar in ipairs(Bikinitools.Automation.queueLockVars)
    do
        if Bikinitools.Options["enable" .. Bikinitools:ucFirst(lockVar)] == true
        then
            return true;
        end
    end

    return false;
end


function Bikinitools.Automation:isOtherJobRunning(currentJob)
    for i, lockVar in ipairs(Bikinitools.Automation.queueLockVars)
    do
        if lockVar ~= currentJob and Bikinitools.Automation[lockVar .. "Working"] == true
        then
            return true;
        end
    end

    return false;
end


function Bikinitools.Automation:stopAll()
	Bikinitools.Automation.retryEnabled = false;
	Bikinitools.Automation.processedSinceLastSort = 0;

	for i, lockVar in ipairs(Bikinitools.Automation.queueLockVars)
	do
		if Bikinitools.Automation[lockVar .. "Working"]
		then
			Bikinitools.Automation["stop" .. Bikinitools:ucFirst(lockVar)](self);
		end
	end
end


function Bikinitools.Automation:checkAllQueues(player)
	for i, lockVar in ipairs(Bikinitools.Automation.queueLockVars)
	do
		if Bikinitools.Options["enable" .. Bikinitools:ucFirst(lockVar)]
		then
			Bikinitools.Automation["check" .. Bikinitools:ucFirst(lockVar) .. "Queue"](self, player);
		end
	end
end


function Bikinitools.Automation:addSquareToQueue(square)
	if square
	then
		table.insert(Bikinitools.Automation.squareProcessingQueue, square);
	end
end


function Bikinitools.Automation:removeFirstSquareFromQueue()
	return table.remove(Bikinitools.Automation.squareProcessingQueue, 1);
end


function Bikinitools.Automation:relocateIfNeeded(square, player)
	local playerSquare = Bikinitools.Square:getCurrent(player or Bikinitools:getPlayer());

	if Bikinitools.Automation.lastRelocateSquare ~= square
	then
		local targetRoom = square:getRoom();

		if square:getZ() ~= playerSquare:getZ() or targetRoom ~= playerSquare:getRoom()
		then
			--Bikinitools:message("relocating to " .. Bikinitools:printSquareCoords(square, true));

			Bikinitools.Automation.lastRelocateSquare = square;

            if targetRoom
            then
                print("targetRoom found");

                Bikinitools.Square:walkTo(player, targetRoom:getRandomFreeSquare());
            else
                print("square fallback");

                Bikinitools.Square:walkTo(player, square);
            end

			return true;
		end
	end

	return false;
end


-- custom queue for processing squares via custom methods


function Bikinitools.Automation:checkSquareProcessingQueue(config)
    if Bikinitools.Automation[config.runLockVariable]
    then
        local queueLength = Bikinitools:countArrayElements(Bikinitools.Automation.squareProcessingQueue);

        if queueLength > 0
        then
			--if not config.disableHighlighting
			--then
				--Bikinitools.Grid:highlightSquares(Bikinitools.Automation.squareProcessingQueue, true);
			--end

            if not ISTimedActionQueue.isPlayerDoingAction(config.player)
            then
				if not config.disableSortByDistance and Bikinitools.Automation.processedSinceLastSort >= Bikinitools.Automation.sortEveryXSquares
				then
					Bikinitools.Automation.squareProcessingQueue = Bikinitools.Grid:sortSquaresByDistanceToSquare(
						Bikinitools.Automation.squareProcessingQueue,
						Bikinitools.Square:getCurrent(config.player)
					);

					Bikinitools.Automation.processedSinceLastSort = 0;
				end

				if config.postSortFunction
				then
					Bikinitools.Automation.squareProcessingQueue = config.postSortFunction(Bikinitools.Automation.squareProcessingQueue);
				end

                local square = Bikinitools.Automation.squareProcessingQueue[1];
                local removeFirst = true;

                if config.removeCheckFunction
                then
                    removeFirst = config.removeCheckFunction(square, config.player);
                end

                if removeFirst == "wait"
                then
                    return true;
                elseif removeFirst == "discard"
                then
					Bikinitools.Automation:removeFirstSquareFromQueue()

                    return true;
                end

                if removeFirst == true
                then
                    square = Bikinitools.Automation:removeFirstSquareFromQueue();
                else
                    square = Bikinitools.Automation.squareProcessingQueue[1];
                end

				Bikinitools.Automation.squareQueueLength = queueLength;

                config.squareFunction.fn(square,
					config.squareFunction.param1,
					config.squareFunction.param2,
					config.squareFunction.param3
				);

				Bikinitools.Automation.ticksSinceLastSquareFn = 0;
				Bikinitools.Automation.processedSinceLastSort = Bikinitools.Automation.processedSinceLastSort + 1;
            end

			Bikinitools.Automation.ticksSinceLastSquareFn = Bikinitools.Automation.ticksSinceLastSquareFn + 1;
        end

		if Bikinitools.Automation.ticksSinceLastSquareFn > 5000
		then
			--Bikinitools:message("queue bugged: removing unreachable square");

			Bikinitools.Automation:removeFirstSquareFromQueue();
		end

        if queueLength == 0
        then
			if Bikinitools.Automation.squareQueueStartSquare
			then
				Bikinitools.Square:walkTo(config.player, Bikinitools.Automation.squareQueueStartSquare);
			end

            config.stopFunction(config.player);

			if Bikinitools.Automation.onComplete
			then
				Bikinitools.Automation.onComplete();
			end

			Bikinitools.Automation.onComplete = nil;
        end
    end
end


function Bikinitools.Automation:retryLastSquareProcessingQueue()
	Bikinitools.Automation:startSquareProcessingQueue(
		Bikinitools.Automation.lastSquareQueueStartParams.items,
		Bikinitools.Automation.lastSquareQueueStartParams.runLockVariable,
		Bikinitools.Automation.lastSquareQueueStartParams.message,
		Bikinitools.Automation.lastSquareQueueStartParams.startSquare,
		Bikinitools.Automation.lastSquareQueueStartParams.onComplete
	);
end


function Bikinitools.Automation:startSquareProcessingQueue(items, runLockVariable, message, startSquare, onComplete)
    --if not Bikinitools.Automation[runLockVariable] and message
    --then
        --Bikinitools:message(message);
    --end

    Bikinitools.Automation[runLockVariable] = true;
    Bikinitools.Automation.squareQueueRunning = true;
	Bikinitools.Automation.ticksSinceLastSquareFn = 0;
	Bikinitools.Automation.processedSinceLastSort = Bikinitools.Automation.sortEveryXSquares;

    if startSquare
    then
        Bikinitools.Automation.squareQueueStartSquare = startSquare;
    end

	if Bikinitools:isFunction(items)
	then
		items = items();
	end

    Bikinitools.Automation.squareProcessingQueue = items;
	Bikinitools.Automation.squareQueueLength = Bikinitools:countArrayElements(items);

	Bikinitools.Automation.onComplete = onComplete;

	Bikinitools.Automation.lastSquareQueueStartParams.items = Bikinitools:copyArray(items);
	Bikinitools.Automation.lastSquareQueueStartParams.runLockVariable = runLockVariable;
	Bikinitools.Automation.lastSquareQueueStartParams.message = message;
	Bikinitools.Automation.lastSquareQueueStartParams.startSquare = startSquare;
	Bikinitools.Automation.lastSquareQueueStartParams.onComplete = onComplete;
end


function Bikinitools.Automation:stopSquareProcessingQueue(runLockVariable, message)
    --if Bikinitools.Automation[runLockVariable] and message
    --then
        --Bikinitools:message(message);
    --end

    Bikinitools.Automation[runLockVariable] = false;
    Bikinitools.Automation.squareQueueRunning = false;
	Bikinitools.Automation.ticksSinceLastSquareFn = 0;

    Bikinitools.Automation.squareQueueStartSquare = nil;
    Bikinitools.Automation.squareProcessingQueue = {};
	Bikinitools.Automation.squareQueueLength = 0;

	if Bikinitools.Automation.onComplete
	then
		Bikinitools.Automation.onComplete();
	end

	Bikinitools.Automation.onComplete = nil;

	setGameSpeed(1);
end


-- context menu helpers


function Bikinitools.Automation:createAutomationContextMenu(context, player)
    if Bikinitools.Automation:isAnyAutomationEnabled()
    then
        if not player:getVehicle()
        then
            Bikinitools:addContextMenuHeader(context, getText("IGUI_bikinitools_O_Context_Automation"));

            local square = Bikinitools.Square:getCurrent(player);
            local squaresBySize = {};

			if Bikinitools.Options:get("enableFunSizeRadii") == true
			then
				Bikinitools.Automation.max3x3radius = 7;
			else
				Bikinitools.Automation.max3x3radius = 3;
			end

            for size = Bikinitools.Automation.min3x3radius, Bikinitools.Automation.max3x3radius
            do
				squaresBySize[size] = function()
					return Bikinitools.Grid:getAllSquaresIn3x3GridRadius(square, size);
				end
            end

            for i, lockVar in ipairs(Bikinitools.Automation.queueLockVars)
            do
				local toolNameUC = Bikinitools:ucFirst(lockVar);

                if Bikinitools.Options["enable" .. toolNameUC]
                then
                    if not Bikinitools.Automation:isOtherJobRunning(lockVar)
                    then
                        Bikinitools.Automation["add" .. toolNameUC .. "ContextMenuEntries"](self, context, player, squaresBySize);
                    else
                        Bikinitools:addDisabledContextMenuItem(context, getText("IGUI_bikinitools_O_Context_" .. toolNameUC) .. " " .. getText("IGUI_bikinitools_A_DisabledLC"));
                    end
                end
            end
        --else
            --Bikinitools:addDisabledContextMenuItem(context, getText("IGUI_bikinitools_O_Context_Automation") .. " " .. Bikinitools:ucFirst(getText("IGUI_bikinitools_A_DisabledLC")) .. ": " .. getText("IGUI_bikinitools_A_VehicleTooClose"));
        end
    end
end