-- config


Bikinitools.Options:register("beReadyForFeatureToggles", false, true);


-- helpers to not drown in if statements


function Bikinitools.Event:needsEnterVehicleEvent()
    return Bikinitools.Options.beReadyForFeatureToggles
        or Bikinitools.Options.enableVehicleItemHide;
end


function Bikinitools.Event:needsExitVehicleEvent()
    return Bikinitools.Options.beReadyForFeatureToggles
        or Bikinitools.Options.enableVehicleItemHide;
end


function Bikinitools.Event:needsPlayerUpdateEvent()
    return Bikinitools.Options.beReadyForFeatureToggles
        or Bikinitools.Automation:isAnyAutomationEnabled();
end


function Bikinitools.Event:needsKeyStartPressedEvent()
    return Bikinitools.Options.beReadyForFeatureToggles
        or Bikinitools.Automation:isAnyAutomationEnabled();
end


-- needed for compatibility with survivors, so events are not fired by them


function Bikinitools.Event:isFiredByPlayer(player)
    return Bikinitools.Event.currentPlayerId == player:getID();
end


-- init all modules as needed


function Bikinitools.Event.initialize()
	if not Bikinitools.Options:get("fullyDisableBT")
	then
		Bikinitools.Event.onRespawn();

		--Events.OnPlayerDeath.Add(Bikinitools.Event.onDeath);
		Events.OnCreatePlayer.Add(Bikinitools.Event.onRespawn);

		if Bikinitools:gameIsMP()
		then
			Bikinitools.Options:disableBrokenInMP();
			--Bikinitools.Options:save();
		end

		Bikinitools.Options:disableObsolete();
		Bikinitools.Options:applyCheatRestrictions();

		if Bikinitools.Event:needsEnterVehicleEvent()
		then
			Events.OnEnterVehicle.Add(Bikinitools.Event.onVehicleEnter);
		end

		if Bikinitools.Event:needsExitVehicleEvent()
		then
			Events.OnExitVehicle.Add(Bikinitools.Event.onVehicleExit);
		end

        Events.OnPlayerUpdate.Add(Bikinitools.Event.onPlayerUpdate);

        if Bikinitools.Options.lowPerformanceQueueMode
        then
            Events.EveryOneMinute.Add(Bikinitools.Event.everyMinute);
        end

		if Bikinitools.Event:needsKeyStartPressedEvent()
		then
			Events.OnKeyStartPressed.Add(Bikinitools.Event.onKeyDown);
		end

		Events.OnServerCommand.Add(Bikinitools.Event.onServerCommand);

		Events.OnFillWorldObjectContextMenu.Add(Bikinitools.Event.onCreateContextMenu);
	end
end


-- prepare event handlers


function Bikinitools.Event.onDeath(player)
    if Bikinitools.Event:isFiredByPlayer(player)
    then
        Bikinitools.Event.currentPlayer = nil;
        Bikinitools.Event.currentPlayerId = nil;
    end
end


function Bikinitools.Event.onRespawn(playerId, player)
    Bikinitools.Event.currentPlayer = player or getPlayer();
    Bikinitools.Event.currentPlayerId = Bikinitools.Event.currentPlayer:getID();
end


function Bikinitools.Event.onCreateContextMenu(player, contextMenu, worldobjects, test)
    player = getSpecificPlayer(player);

    if Bikinitools.Event:isFiredByPlayer(player)
    then
		if Bikinitools:gameIsMP()
		then
			local isAdmin = Bikinitools:playerIsAdmin();

			if (isAdmin and Bikinitools.Config.lastLoadedSettingsFile == Bikinitools.Config.restrictedSettingsFile)
				or (not isAdmin and Bikinitools.Config.lastLoadedSettingsFile == Bikinitools.Config.settingsFile)
			then
				Bikinitools.Options:load();
			end
		end

		local vehicle = Bikinitools.Vehicle:get(player, true);
		local btMenu = Bikinitools:addContextMenuSubMenu(contextMenu,
			"bikinitools",
			nil, nil, {
				icon = true
			}
		);

		Bikinitools.Vehicle:createSpecificVehicleContextMenu(btMenu, player, vehicle);
		Bikinitools.Automation:createAutomationContextMenu(btMenu, player);
		Bikinitools.Vehicle:createVehicleContextMenu(btMenu, player);
		Bikinitools.Options.createConfigContextMenu(btMenu, player);
    end
end


function Bikinitools.Event.everyMinute()
	local player = Bikinitools:getPlayer();

    if Bikinitools.Event:isFiredByPlayer(player)
    then
		if Bikinitools.Options.lowPerformanceQueueMode
		then
			Bikinitools.Automation:checkAllQueues(player);
		end
    end
end


function Bikinitools.Event.onPlayerUpdate(player)
    if Bikinitools.Event:isFiredByPlayer(player)
    then
        if not Bikinitools.Options.lowPerformanceQueueMode
		then
			Bikinitools.Automation:checkAllQueues(player);
		end

		if Bikinitools.Automation:isAnyAutomationEnabled()
		then
			Bikinitools.Grid:highlightSquares(Bikinitools.Automation.squareProcessingQueue, true);
		end

        local currentTime = Calendar.getInstance():getTimeInMillis();

        if #Bikinitools.messageQueue > 0 and currentTime - Bikinitools.msBetweenMessages > Bikinitools.lastMessageAt
        then
            table.remove(Bikinitools.messageQueue, 1)();

            Bikinitools.lastMessageAt = currentTime;
        end
    end
end


function Bikinitools.Event.onVehicleEnter(player)
    if Bikinitools.Event:isFiredByPlayer(player)
    then
        Bikinitools.Vehicle.playerVehicle = Bikinitools.Vehicle:get(player, true);

        if Bikinitools.Options.enableVehicleItemHide
        then
            Bikinitools.Item:putAwayEquippedItems(player);
        end
    end
end


function Bikinitools.Event.onVehicleExit(player)
    if Bikinitools.Event:isFiredByPlayer(player)
    then
        Bikinitools.Vehicle.playerVehicle = nil;

        if Bikinitools.Options.enableVehicleItemHide
        then
            Bikinitools.Item:restoreEquippedItems(player);
        end
    end
end


function Bikinitools.Event.onKeyDown(key)
    if Bikinitools.Automation.squareQueueRunning and Bikinitools.Event:isFiredByPlayer(Bikinitools:getPlayer())
    then
        local stopActions = false;

        for i, keyCode in ipairs(Bikinitools.Automation.queueStopKeys)
        do
            if key == keyCode
            then
                Bikinitools.Automation:stopAll();

				break;
            end
        end
    end
end


function Bikinitools.Event.onServerCommand(moduleName, command, args)
	--print("SERVER COMMAND: " .. moduleName .. " -> " .. command);

	if moduleName == 'bikinitools_client'
	then
		--Bikinitools:message(moduleName .. " | " .. command);

		if command == "reloadVehicleSkin"
		then
			local vehicle = getVehicleById(args["_vehicleId"]);

			if vehicle
			then
				vehicle:setSkinIndex(args["skinIndex"]);
				Bikinitools.Vehicle:reloadSkin(vehicle);
			--else
				--Bikinitools:message("vehicle not found");
			end
		end
	end
end


-- initialize the toolkit


Events.OnGameStart.Add(Bikinitools.Event.initialize);