-- config


Bikinitools.Options:register("enableContainerUnloader", true);
Bikinitools.Automation:register("containerUnloader");

Bikinitools.Automation.lastContainer = nil;


-- methods


function Bikinitools.Automation.startContainerUnloader(player, squares, startSquare, onComplete)
	if Bikinitools:isFunction(squares)
	then
		squares = squares();
	end

    Bikinitools.Automation:startSquareProcessingQueue(
        Bikinitools.Automation:getContainerUnloaderTiles(squares),
        "containerUnloaderWorking",
        getText("IGUI_bikinitools_O_Context_ContainerUnloader") .. " " .. getText("IGUI_bikinitools_A_EnabledLC"),
        startSquare,
		onComplete
    );
end


function Bikinitools.Automation:stopContainerUnloader()
    Bikinitools.Automation:stopSquareProcessingQueue(
        "containerUnloaderWorking",
        getText("IGUI_bikinitools_O_Context_ContainerUnloader") .. " " .. getText("IGUI_bikinitools_A_DisabledLC")
    );
end


function Bikinitools.Automation:checkContainerUnloaderQueue(player)
    Bikinitools.Automation:checkSquareProcessingQueue({
		player = player,
		runLockVariable = "containerUnloaderWorking",
		squareFunction = {
			fn = function(square, player)
				local object = Bikinitools.Square:getFirstFilledContainer(square);

				if object
				then
					Bikinitools.Item:emptyContainerToFloor(object, player);

					Bikinitools.Automation.lastContainer = object;
				end
			end,
			param1 = player
		},
		removeCheckFunction = function(square, player)
			if Bikinitools.Automation:relocateIfNeeded(square, player)
			then
				return "wait";
			end

			local container = Bikinitools.Square:getFirstFilledContainer(square);

			if not container or container == Bikinitools.Automation.lastContainer
				--or not luautils.walkToContainer(container:getContainer(), player:getPlayerNum())
			then
				Bikinitools:message(getText("IGUI_bikinitools_A_UnableToEmptyContainer"));

				return "discard";
			end

			return container ~= nil;
		end,
		stopFunction = function()
			Bikinitools.Automation:stopContainerUnloader()
		end,
	});
end


function Bikinitools.Automation:getContainerUnloaderTiles(squares)
	if Bikinitools:isFunction(squares)
	then
		squares = squares();
	end

	return Bikinitools.Grid:filterSquaresByHasFilledContainer(squares);
end


function Bikinitools.Automation:addContainerUnloaderContextMenuEntries(contextMenu, player, squaresBySize)
    local square = Bikinitools.Square:getCurrent(player);
	local room = square:getRoom();
    local containerUnloaderMenu = Bikinitools:addContextMenuSubMenu(contextMenu,
        getText("IGUI_bikinitools_O_Context_ContainerUnloader")
    );
	local squareCount = 0;

    for radius = Bikinitools.Automation.min3x3radius, Bikinitools.Automation.max3x3radius
    do
		if Bikinitools.Options:get("predetermineAreaViability")
		then
			squareCount = Bikinitools:countArrayElements(Bikinitools.Automation:getContainerUnloaderTiles(squaresBySize[radius]));
		end

        Bikinitools:addContextMenuItem(containerUnloaderMenu,
            getText("IGUI_bikinitools_A_UnloadContainersInArea",
                getText("IGUI_bikinitools_A_Radius" .. radius)
            ),
            Bikinitools.Automation.startContainerUnloader, {player, squaresBySize[radius], square},
            {
                disabled = Bikinitools.Options:get("predetermineAreaViability") and squareCount < 1
            }
        );
    end

	if room
    then
		local building = room:getBuilding();

        Bikinitools:addContextMenuItem(containerUnloaderMenu,
            getText("IGUI_bikinitools_A_UnloadContainersInRoom"),
            Bikinitools.Automation.startContainerUnloader, {
				player,
				function()
					return Bikinitools.Grid:getAllSquaresInRoom(square:getRoom(), "hasFilledContainer");
				end,
				square
			}
        );

		if building
		then
			Bikinitools:addContextMenuItem(containerUnloaderMenu,
				getText("IGUI_bikinitools_A_UnloadContainersInBuilding"),
				Bikinitools.Automation.startContainerUnloader, {
					player,
					function()
						return Bikinitools.Grid:getAllSquaresInBuilding(building, "hasFilledContainer");
					end,
					square
				}
			);
		else
			Bikinitools:addDisabledContextMenuItem(containerUnloaderMenu, Bikinitools:ucFirst(getText("IGUI_bikinitools_A_DisabledLC")) .. ": " .. getText("IGUI_bikinitools_A_NotInBuilding"));
		end
    else
        Bikinitools:addDisabledContextMenuItem(containerUnloaderMenu, Bikinitools:ucFirst(getText("IGUI_bikinitools_A_DisabledLC")) .. ": " .. getText("IGUI_bikinitools_A_NotInRoom"));
    end
end