-- config


Bikinitools.Options:register("enableCorpseStacker", true);
Bikinitools.Automation:register("corpseStacker");


-- methods


function Bikinitools.Automation.startCorpseStacker(player, squares, startSquare)
	if Bikinitools:isFunction(squares)
	then
		squares = squares();
	end

    Bikinitools.Automation:startSquareProcessingQueue(
        Bikinitools.Automation:getCorpseStackerTiles(squares),
        "corpseStackerWorking",
        getText("IGUI_bikinitools_O_Context_CorpseStacker") .. " " .. getText("IGUI_bikinitools_A_EnabledLC"),
        startSquare
    );

	Bikinitools.Automation:addSquareToQueue(startSquare);
end


function Bikinitools.Automation:stopCorpseStacker()
    Bikinitools.Automation:stopSquareProcessingQueue(
        "corpseStackerWorking",
        getText("IGUI_bikinitools_O_Context_CorpseStacker") .. " " .. getText("IGUI_bikinitools_A_DisabledLC")
    );
end


function Bikinitools.Automation:checkCorpseStackerQueue(player)
    local inventory = player:getInventory();

    Bikinitools.Automation:checkSquareProcessingQueue({
		player = player,
		runLockVariable = "corpseStackerWorking",
		disableSortByDistance = true,
		squareFunction = {
            fn = function(square, player)
                if square == Bikinitools.Automation.squareQueueStartSquare
                then
                    Bikinitools.Automation:stopCorpseStacker();

                    return true;
                end

                local corpses = Bikinitools.Square:getValidCorpses(square);

                if #corpses > 0
                then
                    ISWorldObjectContextMenu.onGrabCorpseItem(nil, corpses[1], player:getPlayerNum());

					Bikinitools.Square:walkTo(player, Bikinitools.Automation.squareQueueStartSquare);
                end
            end,
            param1 = player
        },
		removeCheckFunction = function(square)
			if inventory:containsType("CorpseMale") or inventory:containsType("CorpseFemale")
			then
				local corpses = player:getInventory():getAllEvalRecurse(Bikinitools.Item.filterFnCorpse);

				for i = 0, corpses:size() - 1
				do
					ISWorldObjectContextMenu.equip(player, player:getPrimaryHandItem(), corpses:get(i), true, true);
					ISTimedActionQueue.add(ISUnequipAction:new(player, corpses:get(i), 50));
				end

				return "wait";
			end

			local corpseCount = Bikinitools.Automation.squareQueueStartSquare:getDeadBodys():size();

			if corpseCount > 0
			then
				Bikinitools:message("#" .. corpseCount);
			end

			return #Bikinitools.Square:getValidCorpses(square) <= 1;
		end,
		stopFunction = function()
			Bikinitools.Automation:stopCorpseStacker()
		end,
	});
end


function Bikinitools.Automation:getCorpseStackerTiles(squares)
	if Bikinitools:isFunction(squares)
	then
		squares = squares();
	end

    return Bikinitools.Grid:filterSquaresByCorpses(squares, true);
end


-- context menu helper


function Bikinitools.Automation:addCorpseStackerContextMenuEntries(contextMenu, player, squaresBySize)
    local square = Bikinitools.Square:getCurrent(player);
	local room = square:getRoom();
    local corpseStackerMenu = Bikinitools:addContextMenuSubMenu(contextMenu,
        getText("IGUI_bikinitools_O_Context_CorpseStacker")
    );
	local squareCount = 0;

    for radius = Bikinitools.Automation.min3x3radius, Bikinitools.Automation.max3x3radius
    do
		if Bikinitools.Options:get("predetermineAreaViability")
		then
			squareCount = Bikinitools:countArrayElements(Bikinitools.Automation:getCorpseStackerTiles(squaresBySize[radius]));
		end

        Bikinitools:addContextMenuItem(corpseStackerMenu,
            getText("IGUI_bikinitools_A_PickUpSoilInArea",
                getText("IGUI_bikinitools_A_CorpsesLC"),
                getText("IGUI_bikinitools_A_Radius" .. radius)
            ),
            Bikinitools.Automation.startCorpseStacker, {player, squaresBySize[radius], square},
            {
                disabled = Bikinitools.Options:get("predetermineAreaViability") and squareCount < 1
            }
        );
    end

	if room
    then
		local building = room:getBuilding();

        Bikinitools:addContextMenuItem(corpseStackerMenu,
            getText("IGUI_bikinitools_A_StackCorpsesInRoom"),
            Bikinitools.Automation.startCorpseStacker, {
				player,
				function()
					return Bikinitools.Grid:getAllSquaresInRoom(square:getRoom(), "hasCorpse", true);
				end,
				square
			}
        );

		if building
		then
			Bikinitools:addContextMenuItem(corpseStackerMenu,
				getText("IGUI_bikinitools_A_StackCorpsesInBuilding"),
				Bikinitools.Automation.startCorpseStacker, {
					player,
					function()
						return Bikinitools.Grid:getAllSquaresInBuilding(building, "hasCorpse", true);
					end,
					square
				}
			);
		else
			Bikinitools:addDisabledContextMenuItem(corpseStackerMenu, Bikinitools:ucFirst(getText("IGUI_bikinitools_A_DisabledLC")) .. ": " .. getText("IGUI_bikinitools_A_NotInBuilding"));
		end
    else
        Bikinitools:addDisabledContextMenuItem(corpseStackerMenu, Bikinitools:ucFirst(getText("IGUI_bikinitools_A_DisabledLC")) .. ": " .. getText("IGUI_bikinitools_A_NotInRoom"));
    end
end