-- config


Bikinitools.Options:register("enableGardener", true);
Bikinitools.Automation:register("gardener");

Bikinitools.Automation.gardenerActions = {};


-- methods


function Bikinitools.Automation.startGardener(player, squares, cutPlants, removeGrass)
	if Bikinitools:isFunction(squares)
	then
		squares = squares();
	end

    Bikinitools.Automation.gardenerActions = {
        removeGrass = removeGrass,
        cutPlants = cutPlants
    };

    Bikinitools.Automation:startSquareProcessingQueue(
        Bikinitools.Automation:getGardenerTiles(squares, removeGrass, cutPlants),
        "gardenerWorking",
        getText("IGUI_bikinitools_O_Context_Gardener") .. " " .. getText("IGUI_bikinitools_A_EnabledLC")
    );
end


function Bikinitools.Automation:stopGardener()
    Bikinitools.Automation:stopSquareProcessingQueue(
        "gardenerWorking",
        getText("IGUI_bikinitools_O_Context_Gardener") .. " " .. getText("IGUI_bikinitools_A_DisabledLC")
    );

    Bikinitools.Automation.gardenerActions = {};
end


function Bikinitools.Automation:checkGardenerQueue(player)
    local inventory = player:getInventory();

    Bikinitools.Automation:checkSquareProcessingQueue({
		player = player,
		runLockVariable = "gardenerWorking",
		squareFunction = {
            fn = function(square, player)
                if Bikinitools.Automation.gardenerActions.removeGrass
                    and Bikinitools.Square:getGrass(square)
                    and Bikinitools.Square:walkTo(player, square, true)
                then
                    ISWorldObjectContextMenu.doRemoveGrass(player, square);
                end

                if Bikinitools.Automation.gardenerActions.cutPlants
                then
                    local cutTool = Bikinitools.Item:getPlantCuttingTool(player);

                    if cutTool
                        and Bikinitools.Square:getPlant(square)
                        and Bikinitools.Square:walkTo(player, square, true)
                    then
                        ISWorldObjectContextMenu.doRemovePlant(player, square);
                    end
                end
            end,
            param1 = player
        },
		removeCheckFunction = function(square)
			local cutTool = Bikinitools.Item:getPlantCuttingTool(player);

			if (Bikinitools.Automation.gardenerActions.removeGrass and Bikinitools.Square:getGrass(square))
				or (Bikinitools.Automation.gardenerActions.cutPlants and cutTool and Bikinitools.Square:getPlant(square))
			then
				return true;
			else
				return "wait";
			end
		end,
		stopFunction = function()
			Bikinitools.Automation:stopGardener();
		end,
	});
end


function Bikinitools.Automation:getGardenerTiles(squares, removeGrass, cutPlants)
    local filter = nil;

	if Bikinitools:isFunction(squares)
	then
		squares = squares();
	end

    if removeGrass and cutPlants
    then
        return Bikinitools.Grid:filterSquaresByHasGrassOrPlants(squares);
    else
        if cutPlants
        then
            return Bikinitools.Grid:filterSquaresByHasPlants(squares);
        else
            return Bikinitools.Grid:filterSquaresByHasGrass(squares);
        end
    end
end


-- context menu helper


function Bikinitools.Automation:addGardenerContextMenuEntries(contextMenu, player, squaresBySize)
    local cutTool = Bikinitools.Item:getPlantCuttingTool(player);
    local gardenerMenu = Bikinitools:addContextMenuSubMenu(contextMenu,
        getText("IGUI_bikinitools_O_Context_Gardener")
    );

    function createMenuItems(headerTKey, optionTKey, removeGrass, cutPlants)
		local squareCount = 0;

        Bikinitools:addContextMenuHeader(gardenerMenu, Bikinitools:ucFirst(getText(headerTKey)));

        for radius = Bikinitools.Automation.min3x3radius, Bikinitools.Automation.max3x3radius
        do
			if Bikinitools.Options:get("predetermineAreaViability")
			then
				squareCount = Bikinitools:countArrayElements(Bikinitools.Automation:getGardenerTiles(squaresBySize[radius], removeGrass, cutPlants));
			end

            Bikinitools:addContextMenuItem(gardenerMenu,
                getText(getText(optionTKey), getText("IGUI_bikinitools_A_Radius" .. radius)),
                Bikinitools.Automation.startGardener, {player, squaresBySize[radius], cutPlants, removeGrass},
                {
                    disabled = Bikinitools.Options:get("predetermineAreaViability") and squareCount < 1
                }
            );
        end
    end

    if cutTool
    then
        createMenuItems("IGUI_bikinitools_A_Plants", "IGUI_bikinitools_A_RemovePlantsInArea", true, true);
    else
        Bikinitools:addDisabledContextMenuItem(gardenerMenu, Bikinitools:ucFirst(getText("IGUI_bikinitools_A_NoCuttingTool")));
    end

    createMenuItems("IGUI_bikinitools_A_Grass", "IGUI_bikinitools_A_RemoveGrassInArea", true, false);

    if cutTool
    then
        createMenuItems("IGUI_bikinitools_A_Shrubs", "IGUI_bikinitools_A_RemoveShrubsInArea", false, true);
    else
        Bikinitools:addDisabledContextMenuItem(gardenerMenu, Bikinitools:ucFirst(getText("IGUI_bikinitools_A_NoCuttingTool")));
    end
end