-- config


Bikinitools.Options:register("enableHomeWrecker", true);
Bikinitools.Options:register("unloadContainersBeforeScrapping", true);
Bikinitools.Options:register("pickUpNailsAfterScrapping", true);
Bikinitools.Options:register("dropElectronicsAfterScrapping", true);
Bikinitools.Automation:register("homeWrecker");

Bikinitools.Automation.lastScrappable = nil;
Bikinitools.Automation.lastContainerSquare = nil;


-- methods


function Bikinitools.Automation.startHomeWrecker(player, squares, startSquare)
	Bikinitools.Automation.retryEnabled = true;

	if Bikinitools:isFunction(squares)
	then
		squares = squares();
	end

	Bikinitools.Automation:startSquareProcessingQueue(
		Bikinitools.Automation:getHomeWreckerTiles(squares),
		"homeWreckerWorking",
		getText("IGUI_bikinitools_O_Context_HomeWrecker") .. " " .. getText("IGUI_bikinitools_A_EnabledLC"),
		startSquare
	);
end


function Bikinitools.Automation:stopHomeWrecker()
	if Bikinitools.Automation.retryEnabled
	then
		Bikinitools.Automation.retryEnabled = false;

		local function startFn()
			Bikinitools.Automation:retryLastSquareProcessingQueue();
		end;

		if Bikinitools.Options.unloadContainersBeforeScrapping and Bikinitools.Options.enableContainerUnloader
		then
			Bikinitools.Automation.startContainerUnloader(
				Bikinitools:getPlayer(),
				Bikinitools.Automation.lastSquareQueueStartParams.items,
				Bikinitools.Automation.lastSquareQueueStartParams.startSquare,
				startFn
			);
		else
			startFn();
		end
	else
		Bikinitools.Automation:stopSquareProcessingQueue(
			"homeWreckerWorking",
			getText("IGUI_bikinitools_O_Context_HomeWrecker") .. " " .. getText("IGUI_bikinitools_A_DisabledLC")
		);
	end
end


function Bikinitools.Automation:checkHomeWreckerQueue(player)
    local inventory = player:getInventory();

	Bikinitools.Automation:checkSquareProcessingQueue({
		player = player,
		runLockVariable = "homeWreckerWorking",
		--disableHighlighting = true,
		--postSortFunction = function(squares)
			--return Bikinitools.Grid:moveSquaresWithDoorsToBeginning(squares);
		--end,
		squareFunction = {
            fn = function(square, player)
				local object = Bikinitools.Square:getFirstScrappableObject(square, player);

				if object
				then
					Bikinitools.Automation.lastScrappable = object.object;

					if object.moveProps:walkToAndEquip(player, square, "scrap")
					then
						Bikinitools:message(Bikinitools:ucFirst(getText("IGUI_bikinitools_A_ScrappingLC")) .. " " .. Translator.getMoveableDisplayName(object.moveProps.name));

						ISTimedActionQueue.add(ISMoveablesAction:new(player, square, object.moveProps, "scrap"));
					end
				end
            end,
            param1 = player
        },
		removeCheckFunction = function(square, player)
			if Bikinitools.Options.dropElectronicsAfterScrapping
			then
				local droppedScrap = Bikinitools.Item:dropElectronicsScrapResults(player);

				if droppedScrap > 0
				then
					Bikinitools:message(getText("IGUI_bikinitools_I_DroppingXScrap", droppedScrap));

					return "wait";
				end
			end

			if Bikinitools.Options.pickUpNailsAfterScrapping and Bikinitools.Square:walkTo(player, square, true)
			then
				local nailsOnFloor = Bikinitools.Item:pickUpNailsFromFloor(player);

				if nailsOnFloor > 0
				then
					Bikinitools:message(getText("IGUI_bikinitools_I_PickingUpXNails", nailsOnFloor));

					return "wait";
				end
			end

			if Bikinitools.Automation:relocateIfNeeded(square, player)
			then
				return "wait";
			end

			local objects = Bikinitools.Square:getPotentiallyScrappableObjects(square, player);
			local count = #objects;

			if count > 0
			then
				if Bikinitools.Options.unloadContainersBeforeScrapping
				then
					local containers = Bikinitools.Square:getFilledContainerObjects(square);

					if #containers > 0
					then
						if Bikinitools.Automation.lastContainerSquare == square
						then
							Bikinitools:message(getText("IGUI_bikinitools_A_UnableToEmptyContainer"));

							return "discard";
						end

						Bikinitools.Square:walkTo(player, square, true);

						for i = 1, #containers
						do
							Bikinitools.Item:emptyContainerToFloor(containers[i], player);
						end

						Bikinitools.Automation.lastContainerSquare = square;

						return "wait";
					end
				end

				local nextScrappable = Bikinitools.Square:getFirstScrappableObject(square, player, false, true);

				if not nextScrappable or Bikinitools.Automation.lastScrappable == nextScrappable
				then
					return "discard";
				end
			else
				return "discard";
			end

			return count <= 1 and nextScrappable ~= nil;
		end,
		stopFunction = function()
			Bikinitools.Automation:stopHomeWrecker();
			Bikinitools.Item:dropElectronicsScrapResults(player);
		end,
	});
end


function Bikinitools.Automation:getHomeWreckerTiles(squares)
	if Bikinitools:isFunction(squares)
	then
		squares = squares();
	end

	return Bikinitools.Grid:filterSquaresByHasScrappable(squares, Bikinitools.Options.unloadContainersBeforeScrapping or false);
end


-- context menu helper


function Bikinitools.Automation:addHomeWreckerContextMenuEntries(contextMenu, player, squaresBySize)
    local square = Bikinitools.Square:getCurrent(player);
	local room = square:getRoom();
    local homeWreckerMenu = Bikinitools:addContextMenuSubMenu(contextMenu,
        getText("IGUI_bikinitools_O_Context_HomeWrecker")
    );
	local squareCount = 0;

    for radius = Bikinitools.Automation.min3x3radius, Bikinitools.Automation.max3x3radius
    do
		if Bikinitools.Options:get("predetermineAreaViability")
		then
			squareCount = Bikinitools:countArrayElements(Bikinitools.Automation:getHomeWreckerTiles(squaresBySize[radius]));
		end

        Bikinitools:addContextMenuItem(homeWreckerMenu,
            getText("IGUI_bikinitools_A_DismantleItemsInArea", getText("IGUI_bikinitools_A_Radius" .. radius)),
            Bikinitools.Automation.startHomeWrecker, {player, squaresBySize[radius], square},
            {
                disabled = Bikinitools.Options:get("predetermineAreaViability") and squareCount < 1
            }
        );
    end

    if room
    then
		local building = room:getBuilding();

        Bikinitools:addContextMenuItem(homeWreckerMenu,
            getText("IGUI_bikinitools_A_DismantleItemsInRoom"),
            Bikinitools.Automation.startHomeWrecker, {
				player,
				function()
					return Bikinitools.Grid:getAllSquaresInRoom(room, "hasScrappable",
						Bikinitools.Options.unloadContainersBeforeScrapping or false
					);
				end,
				square
			}
        );

		if building
		then
			Bikinitools:addContextMenuItem(homeWreckerMenu,
				getText("IGUI_bikinitools_A_DismantleItemsInBuilding"),
				Bikinitools.Automation.startHomeWrecker, {
					player,
					function()
						return Bikinitools.Grid:getAllSquaresInBuilding(building, "hasScrappable",
							Bikinitools.Options.unloadContainersBeforeScrapping or false
						);
					end,
					square
				}
			);
		else
			Bikinitools:addDisabledContextMenuItem(homeWreckerMenu, Bikinitools:ucFirst(getText("IGUI_bikinitools_A_DisabledLC")) .. ": " .. getText("IGUI_bikinitools_A_NotInBuilding"));
		end
    else
        Bikinitools:addDisabledContextMenuItem(homeWreckerMenu, Bikinitools:ucFirst(getText("IGUI_bikinitools_A_DisabledLC")) .. ": " .. getText("IGUI_bikinitools_A_NotInRoom"));
    end
end