-- config


Bikinitools.Options:register("enableLumberjack", true);
Bikinitools.Automation:register("lumberjack");


-- methods


function Bikinitools.Automation.startLumberjack(player, squares)
	if Bikinitools:isFunction(squares)
	then
		squares = squares();
	end

    Bikinitools.Automation:startSquareProcessingQueue(
        Bikinitools.Automation:getLumberjackTiles(squares),
        "lumberjackWorking",
        getText("IGUI_bikinitools_O_Context_Lumberjack") .. " " .. getText("IGUI_bikinitools_A_EnabledLC")
    );
end


function Bikinitools.Automation:stopLumberjack()
    Bikinitools.Automation:stopSquareProcessingQueue(
        "lumberjackWorking",
        getText("IGUI_bikinitools_O_Context_Lumberjack") .. " " .. getText("IGUI_bikinitools_A_DisabledLC")
    );
end


function Bikinitools.Automation:checkLumberjackQueue(player)
    local inventory = player:getInventory();

	Bikinitools.Automation:checkSquareProcessingQueue({
		player = player,
		runLockVariable = "lumberjackWorking",
		squareFunction = {
            fn = function(square, player)
                local tree = square:getTree();

                if tree and Bikinitools.Square:walkTo(player, square, true)
                then
                    ISWorldObjectContextMenu.doChopTree(player, tree);
                end
            end,
            param1 = player
        },
		stopFunction = function()
			Bikinitools.Automation:stopLumberjack()
		end,
	});
end


function Bikinitools.Automation:getLumberjackTiles(squares)
	if Bikinitools:isFunction(squares)
	then
		squares = squares();
	end

    return Bikinitools.Grid:filterSquaresByHasTree(squares);
end


-- context menu helper


function Bikinitools.Automation:addLumberjackContextMenuEntries(contextMenu, player, squaresBySize)
    local axe = Bikinitools.Item:getBestWoodChoppingAxe(player);
    local lumberjackMenu = Bikinitools:addContextMenuSubMenu(contextMenu,
        getText("IGUI_bikinitools_O_Context_Lumberjack")
    );

    if axe
    then
		local squareCount = 0;

        for radius = Bikinitools.Automation.min3x3radius, Bikinitools.Automation.max3x3radius
        do
            if Bikinitools.Options:get("predetermineAreaViability")
			then
				squareCount = Bikinitools:countArrayElements(Bikinitools.Automation:getLumberjackTiles(squaresBySize[radius]));
			end

            Bikinitools:addContextMenuItem(lumberjackMenu,
                getText("IGUI_bikinitools_A_ChopTreesInArea", getText("IGUI_bikinitools_A_Radius" .. radius)),
                Bikinitools.Automation.startLumberjack, {player, squaresBySize[radius]},
                {
                    disabled = Bikinitools.Options:get("predetermineAreaViability") and squareCount < 1
                }
            );
        end
    else
        Bikinitools:addDisabledContextMenuItem(lumberjackMenu, Bikinitools:ucFirst(getText("IGUI_bikinitools_A_DisabledLC")) .. ": " .. getText("IGUI_bikinitools_A_NoAxe"));
    end
end