-- config


Bikinitools.Options:register("enableVehicleItemHide", false, true);
Bikinitools.Options:register("putHiddenItemsInBackpack", true);

Bikinitools.Item.lastEquippedPrimary = nil;
Bikinitools.Item.lastEquippedOffhand = nil;


-- methods


function Bikinitools.Item:notifyAboutEquippedItems(primaryItem, offhandItem, firstStr, secondStr)
    local notifyItems = {};

    if primaryItem and offhandItem and primaryItem == offhandItem
    then
        table.insert(notifyItems, primaryItem);
    else
        if primaryItem
        then
            table.insert(notifyItems, primaryItem);
        end

        if offhandItem
        then
            table.insert(notifyItems, offhandItem);
        end
    end

    for i, item in ipairs(notifyItems)
    do
        Bikinitools:message((firstStr or "") .. item:getDisplayName() .. (secondStr or ""));
    end
end


function Bikinitools.Item:putAwayEquippedItems(player)
    local primaryItem = player:getPrimaryHandItem();
    local offhandItem = player:getSecondaryHandItem();
    local hotbar = getPlayerHotbar(player:getPlayerNum());
    local primaryInHotbar = hotbar:isInHotbar(primaryItem);
    local offhandInHotbar = hotbar:isInHotbar(offhandItem);

    Bikinitools.Item.lastEquippedPrimary = primaryItem;
    Bikinitools.Item.lastEquippedOffhand = offhandItem;

    -- any action seems to be able to override hand models, so maybe create a dummy action to access methods
    -- local action = ISBaseTimedAction:new(character) <- this one for example
    -- action:setOverrideHandModels(nil, nil)
    -- action:setOverrideHandModels(Bikinitools.Item.lastEquippedPrimary, Bikinitools.Item.lastEquippedOffhand)

    if primaryItem and offhandItem and primaryItem == offhandItem and primaryInHotbar
    then
        hotbar:equipItem(primaryItem);
    else
        if primaryInHotbar
        then
            hotbar:equipItem(primaryItem);
        end

        if offhandInHotbar
        then
            hotbar:equipItem(offhandItem);
        end
    end

    if Bikinitools.Options.putHiddenItemsInBackpack
    then
        if primaryItem and offhandItem and primaryItem == offhandItem and not primaryInHotbar
        then
            Bikinitools.Item:tryToPutItemAway(player, primaryItem);
        else
            if primaryItem and not primaryInHotbar
            then
                Bikinitools.Item:tryToPutItemAway(player, primaryItem);
            end

            if offhandItem and not offhandInHotbar
            then
                Bikinitools.Item:tryToPutItemAway(player, offhandItem);
            end
        end
    end

    player:setPrimaryHandItem(nil);
    player:setSecondaryHandItem(nil);

    Bikinitools.Item:notifyAboutEquippedItems(
        Bikinitools.Item.lastEquippedPrimary,
        Bikinitools.Item.lastEquippedOffhand,
        getText("IGUI_bikinitools_I_PutAwayMy") .. " ",
        " " .. getText("IGUI_bikinitools_I_ForLater")
    );
end


function Bikinitools.Item:restoreEquippedItems(player)
    Bikinitools.Item:notifyAboutEquippedItems(
        Bikinitools.Item.lastEquippedPrimary,
        Bikinitools.Item.lastEquippedOffhand,
        getText("IGUI_bikinitools_I_EquippingMy") .. " "
    );

    local function moveAndEquip(player, itemVarName, equipFn)
        if Bikinitools.Item[itemVarName]:getContainer() ~= player:getInventory()
        then
            player:getInventory():addItem(Bikinitools.Item[itemVarName]);
        end

        equipFn(Bikinitools.Item[itemVarName]);

        Bikinitools.Item[itemVarName] = nil;
    end

    if Bikinitools.Item.lastEquippedPrimary and Bikinitools.Item.lastEquippedOffhand and Bikinitools.Item.lastEquippedPrimary == Bikinitools.Item.lastEquippedOffhand
    then
        if Bikinitools.Options.putHiddenItemsInBackpack
        then
            moveAndEquip(player, "lastEquippedPrimary", function(item)
                player:setPrimaryHandItem(item);
                player:setSecondaryHandItem(item);
            end);
        else
            Bikinitools.Item.lastEquippedPrimary = nil;
        end

        Bikinitools.Item.lastEquippedOffhand = nil;
    else
        if Bikinitools.Item.lastEquippedPrimary
        then
            if Bikinitools.Options.putHiddenItemsInBackpack
            then
                moveAndEquip(player, "lastEquippedPrimary", function(item)
                    player:setPrimaryHandItem(item);
                end);
            else
                Bikinitools.Item.lastEquippedPrimary = nil;
            end
        end

        if Bikinitools.Item.lastEquippedOffhand
        then
            if Bikinitools.Options.putHiddenItemsInBackpack
            then
                moveAndEquip(player, "lastEquippedOffhand", function(item)
                    player:setSecondaryHandItem(item);
                end);
            else
                Bikinitools.Item.lastEquippedOffhand = nil;
            end
        end
    end
end