function Bikinitools.Options:disableContextItemByText(contextMenu, name)
	for i,option in ipairs(contextMenu.options)
	do
		if option.name == name
		then
			contextMenu.options[i].notAvailable = true;

			return true;
		end
	end
end


function Bikinitools.Options:whyNotZoidberg()
    return not getActivatedMods():contains("CheatMenuRB") and Bikinitools.Options:get("enableIconOverride");
end


function Bikinitools:addContextMenuHeader(context, text)
    return Bikinitools:addContextMenuItem(context,
        text, nil, nil, {
            disabled = true,
            icon = true
        }
    );
end


function Bikinitools:addDisabledContextMenuItem(context, text)
    return Bikinitools:addContextMenuItem(context,
        text, nil, nil, {
            disabled = true
        }
    );
end


function Bikinitools:addContextMenuItem(context, text, handler, params, options)
    return Bikinitools:createContextMenuOption(context, text, handler, params, Bikinitools:mergeArrays({
        isSubMenu = false
    }, options));
end


function Bikinitools:addContextMenuSubMenu(context, text, handler, params, options)
    return Bikinitools:createContextMenuOption(context, text, handler, params, Bikinitools:mergeArrays({
        isSubMenu = true
    }, options));
end


function Bikinitools:createContextMenuOption(context, text, handler, params, options)
    -- C:\Program Files (x86)\Steam\steamapps\common\ProjectZomboid\media\lua\client\ISUI\ISContextMenu.lua
    -- ISContextMenu:addOption(name, target, onSelect, param1, param2, param3, param4, param5, param6, param7, param8, param9, param10)

    if not params
    then
        params = {};
    end

    local entry = context:addOption(
        text or nil,
        params[1] or nil,
        handler or nil,
        params[2] or nil,
        params[3] or nil,
        params[4] or nil,
        params[5] or nil,
        params[6] or nil
    );

    entry.checkMark = options.icon;
    entry.notAvailable = options.disabled;

    if options.tip ~= nil and string.sub(options.tip, 0, 5) ~= "IGUI_"
    then
        entry.toolTip = ISWorldObjectContextMenu.addToolTip();
        entry.toolTip.description = options.tip;
    end

    if context.parent == nil
    then
        if Bikinitools.Options:whyNotZoidberg()
        then
            context.tickTexture = getTexture(Bikinitools.Config.icons.zoidbergM);
        else
            entry.checkMark = nil;
        end
    end

    if options.isSubMenu
    then
        local menu = ISContextMenu:getNew(context);

        --if Bikinitools.Options:whyNotZoidberg()
        --then
            local tickTexture = Bikinitools.Config.icons.arrow2;

            if options.tickTexture and type(options.tickTexture) == "string"
            then
                tickTexture = options.tickTexture;
            end

            menu.tickTexture = getTexture(tickTexture);
        --end

        if Bikinitools.Options.recolorContextMenu
        then
            menu.backgroundColor = {
                r = Bikinitools.Config.colors.contextR,
                g = Bikinitools.Config.colors.contextG,
                b = Bikinitools.Config.colors.contextB,
                a = 0.7
            };
            menu.backgroundColorMouseOver = {
                r = Bikinitools.Config.colors.contextR,
                g = Bikinitools.Config.colors.contextG,
                b = Bikinitools.Config.colors.contextB,
                a = 1
            };
            menu.borderColor = {
                r = Bikinitools.Config.colors.highlightR,
                g = Bikinitools.Config.colors.highlightG,
                b = Bikinitools.Config.colors.highlightB,
                a = 1
            };
        end

        context:addSubMenu(entry, menu);

        return menu;
    else
        return entry;
    end
end