-- config


Bikinitools.Options:register("enableSkinSwitcher", false);
Bikinitools.Options:register("useSkinNamesNotIndices", true);


-- methods


function Bikinitools.Vehicle.setSkinIndex(vehicle, newSkinIndex)
    if vehicle and newSkinIndex > -1
    then
		local version = getCore():getGameVersion();
        local olderVersion = version:getMajor() == 41 and version:getMinor() < 69;
		
		if olderVersion
		then
			print("using my own command");
			
			Bikinitools:sendClientCommand("bikinitools", "setVehicleSkinIndex", {
				["_vehicleId"] = vehicle:getId(),
				skinIndex = newSkinIndex
			});
		else
			print("using new command");
			
			Bikinitools:sendClientCommand("vehicle", "setSkinIndex", {
				vehicle = vehicle:getId(),
				index = newSkinIndex
			});
		end

		local skinName = Bikinitools.Vehicle:getSkinNameByIndex(vehicle, newSkinIndex);

        Bikinitools:message(getText("IGUI_bikinitools_V_SwitchingTo") .. " " .. Bikinitools:ternary(
			skinName ~= nil,
			skinName,
			" " .. (newSkinIndex + 1) .. "/" .. vehicle:getSkinCount()
		));
    end
end


function Bikinitools.Vehicle.setNextSkin(vehicle)
    Bikinitools.Vehicle:rotateSkin(vehicle, 1);
end


function Bikinitools.Vehicle.setPreviousSkin(vehicle)
    Bikinitools.Vehicle:rotateSkin(vehicle, -1);
end


function Bikinitools.Vehicle:rotateSkin(vehicle, diff)
    if vehicle
    then
        local newSkinIndex = vehicle:getSkinIndex() + diff;
        local skinCount = vehicle:getSkinCount();

        if diff < 0 and newSkinIndex < 0
        then
            Bikinitools:message(getText("IGUI_bikinitools_V_SkinJumpToEnd") .. " [>>]");

            newSkinIndex = skinCount - 1;
        end

        if diff > 0 and newSkinIndex >= skinCount
        then
            Bikinitools:message("[<<] " .. getText("IGUI_bikinitools_V_SkinJumpToStart"));

            newSkinIndex = 0;
        end

        if skinCount == 1
        then
            Bikinitools:message("[!] " .. getText("IGUI_bikinitools_V_NoSkinAvailable"));

            newSkinIndex = 0;
        end

        Bikinitools.Vehicle.setSkinIndex(vehicle, newSkinIndex);
    end
end


function Bikinitools.Vehicle:getSkinNameByIndex(vehicle, skinIndex)
	if Bikinitools.Options.useSkinNamesNotIndices
	then
		local skin = vehicle:getScript():getSkin(skinIndex);

		if skin
		then
			local texture = Bikinitools:getFieldValue(skin, "texture");

			if texture
			then
                local name = tostring(texture);

                --print("texture at index " .. tostring(skinIndex) .. " is named " .. name);

                if string.find(name, "/", 1, true)
                then
                    name = Bikinitools:getStringFromLastOccurenceOfChar(name, "/");
                end

                name = name:gsub('"', "");

                --print(" -> " .. tostring(name));

				return name;
			end
		end
	end

	return nil;
end


function Bikinitools.Vehicle:skinChangeIsBlacklisted(vehicle)
    return Bikinitools:gameIsMP() and Bikinitools:valueIsInArray(Bikinitools.Config.disableSkinChange, vehicle:getScript():getFullName());
end


-- context menu helper


function Bikinitools.Vehicle:addSkinContextMenuEntry(contextMenu, attachedObject, vehicle)
    local skinMenu = Bikinitools:addContextMenuSubMenu(contextMenu,
        getText("IGUI_bikinitools_V_ChangeSkin")
    );
    local skinCount = vehicle:getSkinCount();
    local isBlacklisted = Bikinitools.Vehicle:skinChangeIsBlacklisted(vehicle);
    local skinName = Bikinitools.Vehicle:getSkinNameByIndex(vehicle, vehicle:getSkinIndex());
    local header = "";

    if not isBlacklisted
    then
        header = getText("IGUI_bikinitools_V_Skin") .. " " .. Bikinitools:ternary(
            skinName ~= nil,
            skinName,
            (vehicle:getSkinIndex() + 1) .. " / " .. vehicle:getSkinCount()
        );
    else
        header = getText("IGUI_bikinitools_O_Context_VehicleSkinSwitcher") .. " " .. getText("IGUI_bikinitools_A_DisabledLC") .. ":";
    end

    header = Bikinitools:addContextMenuHeader(skinMenu, header);

    if skinCount > 1 and not isBlacklisted
    then
        Bikinitools:addContextMenuItem(skinMenu,
            "[<<] " .. getText("IGUI_bikinitools_V_PrevSkin"),
            Bikinitools.Vehicle.setPreviousSkin, {
                vehicle
            }
        );

        Bikinitools:addContextMenuItem(skinMenu,
            getText("IGUI_bikinitools_V_NextSkin") .. " [>>]",
            Bikinitools.Vehicle.setNextSkin, {
                vehicle
            }
        );

        local specificSkinMenu = Bikinitools:addContextMenuSubMenu(skinMenu,
            getText("IGUI_bikinitools_V_SelectSkin")
        );

        for skinIndex = 0, skinCount - 1
        do
			skinName = Bikinitools.Vehicle:getSkinNameByIndex(vehicle, skinIndex);

            Bikinitools:addContextMenuItem(specificSkinMenu,
				Bikinitools:ternary(
					skinName ~= nil,
					skinName,
					getText("IGUI_bikinitools_V_SwitchToSkinNo") .. " " .. (skinIndex + 1)
				),
				Bikinitools.Vehicle.setSkinIndex, {
                    vehicle,
                    skinIndex
                }
            );
        end
    else
        if not isBlacklisted
        then
            Bikinitools:addDisabledContextMenuItem(skinMenu, getText("IGUI_bikinitools_V_NoSkinAvailable"));
        else
            Bikinitools:addDisabledContextMenuItem(skinMenu, getText("IGUI_bikinitools_O_ModIncompatibility"));
        end
    end
end