-- config


Bikinitools.Options:register("enableVehicleSpawner", false);
Bikinitools.Options:register("spawnVehiclesFullyRepaired", false);
Bikinitools.Options:register("spawnVehiclesWithKey", false);

Bikinitools.Vehicle.spawnList = nil;
Bikinitools.Vehicle.spawnerBlacklist = {
    "isoContainer1"
};


-- methods


function Bikinitools.Vehicle:getAvailableScripts()
    local scripts = getScriptManager():getAllVehicleScripts();
    local sorted = {
        normal = {},
        burnt = {},
        wrecked = {}
    };
    local splitStr = "#";

    for i = 1, scripts:size()
    do
        local script = scripts:get(i - 1);
        local name = script:getName();
        local fullName = script:getFullName();
        local displayName = Bikinitools.Vehicle:getName(name);
        local targetFld = "normal";

        if not Bikinitools:valueIsInArray(Bikinitools.Vehicle.spawnerBlacklist, name)
        then
            if script:getPartCount() == 0
            then
                displayName = Bikinitools:trimString(string.gsub(displayName, "Burnt", ""));
                targetFld = "burnt";
            elseif string.match(name, "Smashed")
            then
                targetFld = "wrecked";
            end

            table.insert(
                sorted[targetFld],
                displayName .. splitStr .. fullName
            );
        end
    end

    for key, items in pairs(sorted)
    do
        Bikinitools:sortArrayByValue(sorted[key], true);
        Bikinitools:splitEachArrayItemBySeparator(sorted[key], splitStr);
    end

    return sorted;
end


function Bikinitools.Vehicle.spawn(scriptName)
	if isClient()
	then
        --print(string.format("/addvehicle %s", scriptName));

		SendCommandToServer(string.format("/addvehicle %s", scriptName));
	else
        --print("addVehicle(" .. scriptName .. ")");

		addVehicle(scriptName);
	end

    local vehicle = Bikinitools.Vehicle:get(nil, true);

    if vehicle and vehicle:getPartCount() > 0
    then
        if Bikinitools.Options.spawnVehiclesFullyRepaired
        then
            Bikinitools.Vehicle:repair(vehicle);
        end

        if Bikinitools.Options.spawnVehiclesWithKey
        then
            Bikinitools.Vehicle.addKeyToInventory(vehicle);
        end
    end
end


function Bikinitools.Vehicle:groupScriptsByLetter(vehicles)
    local vehiclesByLetter = {};

    for vIndex, vehicle in ipairs(vehicles)
    do
        local letter = "?";

        if string.match(vehicle[1], "IGUI_")
        then
            letter = string.upper(vehicle[2]);
        else
            letter = vehicle[1];
        end

        letter = Bikinitools:getFirstLetter(letter);

        if letter == "?"
        then
            letter = Bikinitools:getFirstLetter(string.upper(vehicle[2]));
        end

        if not vehiclesByLetter[letter]
        then
            vehiclesByLetter[letter] = {};
        end

        table.insert(vehiclesByLetter[letter], vehicle);
    end

    return Bikinitools:sortArrayByKey(vehiclesByLetter);
end


-- context menu helper


function Bikinitools.Vehicle:addVehicleSpawnerContextMenuEntry(contextMenu, attachedObject)
    if Bikinitools.Vehicle.spawnList == nil
    then
        Bikinitools.Vehicle.spawnList = Bikinitools.Vehicle.getAvailableScripts();
    end

    local spawnMenu = Bikinitools:addContextMenuSubMenu(contextMenu,
        getText("IGUI_bikinitools_V_SpawnVehicle")
    );

    for vType, vehicles in pairs(Bikinitools.Vehicle.spawnList)
    do
        local typeMenu = Bikinitools:addContextMenuSubMenu(spawnMenu,
            getText("IGUI_bikinitools_V_SpawnerCat" .. Bikinitools:ucFirst(vType))
        );
        local vehiclesByLetter = Bikinitools.Vehicle:groupScriptsByLetter(vehicles);
        local menusByLetter = {};

        for letter, vehicles in pairs(vehiclesByLetter)
        do
            if not menusByLetter[letter]
            then
                menusByLetter[letter] = Bikinitools:addContextMenuSubMenu(typeMenu,
                    letter
                );
            end

            table.sort(vehicles, function(a, b)
                return not string.sort(a[1], b[1]);
            end);

            for i, vehicle in ipairs(vehicles)
            do
                Bikinitools:addContextMenuItem(menusByLetter[letter],
                    vehicle[1] .. " [" .. vehicle[2] .. "]",
                    Bikinitools.Vehicle.spawn, {
                        vehicle[2]
                    }
                );
            end
        end
    end
end